# -*- coding: utf-8 -*-

"""DOCUMENT ME"""


import math
from math import pi
import time
from .trajectory import Trajectory, Point, Movement

IDLER    = 0
DEFENDER = 1
ATTACKER = 2
CATCHER  = 3

class Khepera:

    """DOCUMENT ME"""
    

    global_khep_id = 0

    def __init__(self,ident, angle=0, coord_x=0, coord_y=0, name=None):
        
        self.angle = angle
        self.coord_x = coord_x
        self.coord_y = coord_y
        
        # status and type are used in various occasions (mainly, in the automata
        # in decision loops)
        self.status = "dead"
        self.type = DEFENDER

        self.target = self

        # self.control = control
        
        self.id = ident
        
        if name is None:
            self.name = u"Dalek "+str(Khepera.global_khep_id)
        else:
            self.name = name

        self.defense_position = Point()



    def rotate_time(self, angle):
        
        """this.rotate_time(angle)
        
        Return estimated rotation time"""        

        # TODO: Ask PION
        return 0.1


    def translate_time(self, distance):
    
        """this.translate_time(distance)
        
        Return estimated translation time"""
    
        # TODO: Ask PION
        return 0.1
        
        
    def update_coordinates(self, info):
    
        """this.update_coordinates(info)
        
        Update internal x,y,H coordinates from a Coordinate.get_info() call"""
        
        data = info["khep_coord"][self.id]
        self.coord_x = data[0]
        self.coord_y = data[1]
        self.angle = data[2]

    def compute_intercepting_move(self, ball):
        
        """this.compute_intercepting_move(ball) -> move

        Return intercepting move as a dictionary"""
        

        # TODO: The following section will need explicit documentation with
        # graphics -- move the comments to a proper doc
        
        # our angle is H + pi/2, H being ball.angle
        move = Movement()
        move.angle = ball.angle + pi/2 # XXX very suspicious
        
        # our dest_x and dest_y are complicated fellows : they verify :
        
        # dest_y = a * dest_x + b -- with a, b such as ball.y = a * ball.x + b
        # dest_y = c * dest_x + d -- with c, d such as self.y = c * self.x + d
        
        # fortunately, with the angle we find :
        
        # a = tan H, b = ball.y - a * ball.x
        # c = tan(H - 3pi/2), d = coord_y - c * coord_x
        
        # lastly (leap of faith here, document this more thoroughly)
        # dest_x = (d - b)/(a - c) -- and dest_y follows
        
        a = math.tan(ball.angle)
        b = ball.coord_y - a * ball.coord_x
        
        c = math.tan(ball.angle - 3*pi/2) # Even more suspicious
        d = self.coord_y - c * self.coord_x
        
        move.coord_x = (d - b)/(a - c)
        move.coord_y = c * move.coord_x + d

        
        # our distance is calculated from dest_x and dest_y
        delta_x = move.coord_x - self.coord_x
        delta_y = move.coord_y - self.coord_y
        move.distance = math.hypot(delta_x, delta_x)

        return move


    def can_intercept(self, move, ball_trajectory):
    
        """this.can_intercept(move, ball_trajectory) -> bool
        
        Return True if the robot will intercept the ball using the 'move'
        trajectory, False otherwise"""


        time_to_intercept = self.rotate_time(move.angle) +\
            self.translate_time(move.distance)
        
        time_to_dest = ball_trajectory.predict_future(move.coord_x,
                                                      move.coord_y)
        
        return time_to_intercept < time_to_dest

    
    # Note : There used to be a self.move(target) method, but it got deprecated
    # when the brigade leader asked for a constant, polling verification of
    # whether we had arrived or not. Now all the moving logic is done outside
    # this object.

    def is_pointing_right(self, target=None):
    
        """this.is_pointing_right(target) -> bool
        
        Return True if the robot is pointing in the right direction (+/- 0.25
        rad), False otherwise."""
        if target is None:
            target = self.target
        angle = self.angle_to(target)
        return (-0.25 <= angle <= 0.25)

    def angle_to(self, target=None):

        """this.angle_to(target=self.target) -> angle

        Return the angle between the front vector of the robot and the robot to
        target vector."""
        if target is None:
            target = self.target

        delta_x = target.coord_y - self.coord_y
        delta_y = target.coord_x - self.coord_x

        angle =  math.atan2(delta_x, delta_y)
        angle -= self.angle
        angle %= (2 * pi)

        assert angle >= 0, angle

        if angle > pi: # XXX BURNED MY EYES
            angle = -2 * pi + angle
        return angle




    def is_at_destination(self, target=None):
        """this.is_at_destination(self, target) -> bool
        
        Return True if the robot has arrived at its destination (+/- 0.1m),
        False otherwise."""
        
        if target is None:
            target = self.target
        dist = math.hypot(target.coord_x - self.coord_x, target.coord_y - self.coord_y)
        return (dist < 0.01)

    def catch(self, mobile):
        if self.type != CATCHER:
            self.type = CATCHER
            self.flash("Trying to catch the Ball")
        self.target = self.compute_intercepting_move(mobile)

    def idle(self):
        if self.type != IDLER:
            self.type = IDLER
            self.flash("Going idle")
        self.target = self

    def attack(self, mobile):
        if self.type != ATTACKER:
            self.type = ATTACKER
            self.flash("Pushing the ball out my territory")
        self.target = mobile

    def defend(self):
        """
        Hen-SHIN ! Turn this robot into a defender and give it a "go back to
        your defense position" target.
        """
        if self.type != DEFENDER:
            self.type = DEFENDER
            self.flash("Going back to my defence position.")
        self.target = self.defense_position
        

    def flash(self, message=u"EXTERMINATE"):

        """this.flash(message)
        
        EX-TER-MINATE"""
        
        print(u"<%s> %s" % (self.name, message))

    def move(self, control):
        """
        Controls robots moves
        """
        if self.is_at_destination():
            if self.status != "stop":
                self.status = "stop"
                control.stop(self.id)
        elif self.is_pointing_right():
            if self.status != "move":
                self.status = "move"
                control.move(self.id, 1)
        else:
            if self.status != "rotate":
                angle = self.angle_to()
                if angle > 0:
                    speed = (angle / (2 * pi)) + 0.5
                else:
                    speed = (angle / (2 * pi)) - 0.5
                self.status = "rotate"
                # BEWARE THE BEAST INSIDE !! control take speed **clockwise*
                control.rotate(self.id, -speed)
