# -*- coding: utf-8 -*-

from unittest import TestCase, main as unittest_main
from scor.but import trajectory, khepera
from scor.but.trajectory import Point

from math import pi

class KheperaTestAll(TestCase):
    
    """Test case for the khepera.Khepera class : 'everything'"""
    

    def test_init_and_update(self):
        khepy = khepera.Khepera(0, 0, 0, 0, "Luffy")
        khepy.update_coordinates({"khep_coord":{0:(1, 1, pi/2)}})
        self.assertEquals(khepy.coord_x, 1)
        self.assertEquals(khepy.coord_y, 1)
        self.assertEquals(khepy.angle, pi/2)
        

    def test_time_estimations(self):
        # TODO: Update this when we have actual data
        khepy = khepera.Khepera(0, 0, 0, 0, "Zoro")
        self.assertEquals(khepy.rotate_time(pi/2), 0.1)
        self.assertEquals(khepy.translate_time(1), 0.1)
        

    def test_point_and_shoot(self):
        # TODO: test for incertitude (+/- 0.1 rad and 0.01 m)
        khepy = khepera.Khepera(0, angle=pi/2, name="Nami")
        targets = (
                    (Point(0, 1),  True),
                    (Point(0, -1), False),
                    (Point(1, -1), False),
                    (Point(1,  0), False),
                  )
        for target, ok in targets:
            self.assertEquals(khepy.is_pointing_right(target), ok)

        targets = (
                    (Point(0, 0),  True),
                    (Point(0, 1), False),
                  )
        for target, ok in targets:
            self.assertEquals(khepy.is_at_destination(target), ok)

    def test_angle_to_zero(self):
        khepy = khepera.Khepera(0, angle=pi/2, name="Nami")
        destination = Point(0, 1)
        self.assertAlmostEqual(khepy.angle_to( destination), 0)

    def test_angle_to_pi(self):
        khepy = khepera.Khepera(0, angle=pi/2, name="Nami")
        destination = Point(0.00000001, -1)
        self.assertAlmostEqual(khepy.angle_to( destination), -pi)

    def test_angle_to_pos(self):
        # XXX this method is bullshit
        khepy = khepera.Khepera(0, angle=pi/2, name="Nami")
        destination = Point(-1, 0)
        self.assertAlmostEqual(khepy.angle_to(destination),  pi/2)

    def test_angle_to_neg(self):
        # XXX this method is bullshit
        khepy = khepera.Khepera(0, angle=pi/2, name="Nami")
        destination = Point(1, 0)
        self.assertAlmostEqual(khepy.angle_to(destination), -pi/2)

    def test_angle_to_limit(self):
        # XXX this method is bullshit
        khepy = khepera.Khepera(0, angle=pi/2, name="Nami")
        destination = Point(-0.0000000001, -1)
        self.assertAlmostEqual(khepy.angle_to(destination), pi)


if __name__ == '__main__':
    unittest_main()
