"""This module contains various class able to display result of scor interfaces"""

import curses
from time import localtime, strftime


from Tkinter import Canvas, Tk
import Tkinter
import ImageTk
import Image

class CursesCoordinateDisplay(object):
    """Display coordinate information"""

    TIME_STRING   = "%8i data Received. Last %s:%05i"
    TIME_FORMAT   = "%H:%M:%S"
    HEADER_STRING = "Object          x           y            anglee"
    BALL_STRING   = "Ball     :    %8s     %8s          "
    BOT_STRING    = "Bot  %3i :    %8s     %8s     %8s"
    def __init__(self):
        self.received = 0
        self.inited = False

    def init(self):
        """Init the cruses screen. make sure to alway call quit() after init()"""
        self.inited = True
        self.received = 0
        self.screen = curses.initscr()
        curses.noecho()
        curses.cbreak()
        self.screen.keypad(1)

    def quit(self):
        """restore terminal state"""
        curses.nocbreak()
        if hasattr(self, 'screen'):
            self.screen.keypad(0)
        curses.echo()
        curses.endwin()

    def feed(self, data):
        """feed the display with new data to display"""
        self.received += 1

        time_string = strftime(self.TIME_FORMAT, localtime(data["timestamp"]))
        param = (self.received, time_string, (data["timestamp"] % 1) * 100000)
        self.screen.addstr(0, 0, self.TIME_STRING % param)

        self.screen.addstr(1, 0, self.HEADER_STRING, curses.A_BOLD)

        param = tuple( "%.5f" % num for num in data["ball_coord"])
        self.screen.addstr(2, 0, self.BALL_STRING % param)

        for index, ident in enumerate(data["khep_coord"]):
            param = (ident,) + tuple( "%.5f" % num
                                      for num in data["khep_coord"][ident])
            self.screen.addstr(3 + index, 0, self.BOT_STRING % param)

        self.screen.refresh()

class TkVideoDisplay(Tk):
    # XXX I'm reading it too

    width = 640
    height = 480

    def __init__(self, video):
        self.video = video
        Tk.__init__(self)
        self.canvas = Canvas(self, width=self.width, height=self.height)
        self.canvas.pack(expand = Tkinter.YES, fill= Tkinter.BOTH)

    def update(self):
        data = self.video.get_frame()
        # assign to current_image to prevent garbage collecting
        self.current_image = ImageTk.PhotoImage(Image.open(data))
        self.clean()
        self.canvas.create_image(self.width/2, self.height/2,
                                 image=self.current_image, tag='image')
        self.after(100, self.update)

    def clean(self):
        for i in self.canvas.find_all():
            self.canvas.delete(i)

    def mainloop(self):
        self.after(100, self.update)
        Tk.mainloop(self)
