from unittest import TestCase, main as unittest_main

from scor.common.mock.users import AutomatedControlClient
from scor.common.mock.providers import MockControl

class AutomatedControlClientTC(TestCase):

    def test_call_basic(self):
        robots_ids = (1,)
        control = MockControl(robots_ids)
        user = AutomatedControlClient(control,robots_ids)
        user.run()
        self.assertNotEquals(control.callstack, [])

    def test_call_all_ids(self):
        robots_ids = set([0, 2, 3, 4, 5, 6, 12, 45])
        control = MockControl(robots_ids)
        user = AutomatedControlClient(control,robots_ids)
        user.run()

        called_ids = set( call["robot_id"] for call in control.callstack)
        self.assertEquals(called_ids, robots_ids)

    def test_call_all_order(self):
        robots_ids = set([1, 2])
        expected_call = set(["move", "rotate", "stop"])
        control = MockControl(robots_ids)
        user = AutomatedControlClient(control,robots_ids)
        user.run()

        called_action = set( call["call"] for call in control.callstack)
        self.assertEquals(called_action, expected_call)

    def test_call_finish_stopped(self):
        robots_ids = set([1, 2])
        expected_call = set(["move", "rotate", "stop"])
        control = MockControl(robots_ids)
        user = AutomatedControlClient(control,robots_ids)
        user.run()

        last_call = {}
        for call in control.callstack:
            last_call[call["robot_id"]] = call["call"]

        for ident in robots_ids:
            self.assertEquals(last_call[ident], "stop")

if __name__ == '__main__':
    unittest_main()
