# -*- coding: utf-8 -*-

from unittest import TestCase, main as unittest_main
from scor.common.mock.providers import MockControl
import math

class ControlTestAll(TestCase):
    
    """Test case for the control.Control class : 'everything'"""
    

    def test_construction_default(self):
        """test construction (immobile robots)"""
        controller = MockControl()

        for khep in controller.khep_list.itervalues():
            self.assertEquals(khep["speed"], 0)
            self.assertEquals(khep["angular_speed"], 0)
            self.assertEquals(khep["left_speed"], 0)
            self.assertEquals(khep["right_speed"], 0)

    def test_construction_with_ids(self):
        """test construction (immobile robots)"""
        robots_ids = [1, 4, 7, 8, 9]
        controller = MockControl(robots_ids)

        self.assertEquals(set(controller.khep_list), set(robots_ids))
            

    def test_forward_move(self):
        
        """test forward move for one robot"""
        
        controller = MockControl()        
        khep = controller.khep_list[0]

        controller.move(0, 1)
        self.assertEquals(khep["speed"], 1)
        self.assertEquals(khep["angular_speed"], 0)
        self.assertEquals(khep["left_speed"], 1)
        self.assertEquals(khep["right_speed"], 1)
        

    def test_rotation(self):
        
        """test rotation for one robot"""
        
        controller = MockControl()
        khep = controller.khep_list[0]
        
        controller.rotate(0, 1)
        self.assertEquals(khep["speed"], 0)
        self.assertEquals(khep["angular_speed"], 1)
        self.assertEquals(khep["left_speed"], 1)
        self.assertEquals(khep["right_speed"], -1)


    def test_stop(self):
        
        """test stop order for one robot"""
        
        controller = MockControl()
        khep = controller.khep_list[0]
        
        controller.stop(0)
        self.assertEquals(khep["speed"], 0)
        self.assertEquals(khep["angular_speed"], 0)
        self.assertEquals(khep["left_speed"], 0)
        self.assertEquals(khep["right_speed"], 0)


    def test_set_speed(self):
        
        """test set_speed with two different speeds for one robot"""
        
        controller = MockControl()
        khep = controller.khep_list[0]
        
        controller.set_speed(0, 1, 0.5)
        self.assertEquals(khep["left_speed"], 1)
        self.assertEquals(khep["right_speed"], 0.5)


    def test_callstack(self):

        """test call stack construction"""

        controller = MockControl()
        
        controller.move(0, 1)
        controller.rotate(0, 1)
        controller.stop(0)
        controller.set_speed(0, 1, 0.5)

        self.assertEquals(controller.callstack[0]["call"], "move")
        self.assertEquals(controller.callstack[0]["robot_id"], 0)
        self.assertEquals(controller.callstack[0]["arglist"], [1])
        self.assertEquals(controller.callstack[1]["call"], "rotate")
        self.assertEquals(controller.callstack[1]["robot_id"], 0)
        self.assertEquals(controller.callstack[1]["arglist"], [1])
        self.assertEquals(controller.callstack[2]["call"], "stop")
        self.assertEquals(controller.callstack[2]["robot_id"], 0)
        self.assertEquals(controller.callstack[2]["arglist"], [])
        self.assertEquals(controller.callstack[3]["call"], "set_speed")
        self.assertEquals(controller.callstack[3]["robot_id"], 0)
        self.assertEquals(controller.callstack[3]["arglist"], [1, 0.5])


if __name__ == '__main__':
    unittest_main()
