import sys

import time

from .client import get_proxy
from .config import get_scor_config, ROBOT_SECTION_MASK
from .mock.users import AutomatedControlClient
from .mock.scor_danse import ScorDanseClient
from .display import CursesCoordinateDisplay, TkVideoDisplay

def run_control_tester(argv=None):
    config, argv = get_scor_config(argv=argv)
    config.set_default_teams()

    if len(argv) > 1 and argv[1] in ('-h', '--help'):
        # TODO improve the help message
        print "This script do several call to the served Control interface"
        sys.exit(0)

    control = get_proxy(config, "control")
    robots_ids = config.robots_ids()
    tester = AutomatedControlClient(control, robots_ids)

    tester.run(delay=1, verbose=True)

def run_danse(argv=None):
    config, argv = get_scor_config(argv=argv)
    config.set_default_teams()

    if len(argv) > 1 and argv[1] in ('-h', '--help'):
        # TODO improve the help message
        print "This script does a super ultra extra danse."
        sys.exit(0)

    control = get_proxy(config, "control")
    robots_ids = config.robots_ids()
    tester = ScorDanseClient(control, robots_ids, config)

    tester.run(delay=5, verbose=True)

def run_coordinate_tester(argv=None):
    config, argv = get_scor_config(argv=argv)
    team = u'south'
    config.set_default_teams()

    if len(argv) > 1:
        if argv[1] in ('-h', '--help'):
            # TODO improve the help message
            print """
            This script display the value returned by a Coordinate instance
            
            options:
            -h --help       print this help
            arguments:
            team     choose team. team must be south (default) or north
            """
            sys.exit(0)
        elif argv[1] == u'north':
            team = argv[1]

    coordinate = get_proxy(config, "coordinate", team)

    display = CursesCoordinateDisplay()

    current_time_stamp = -1
    try:
        display.init()
        while 1:
            data = coordinate.get_info(current_time_stamp)
            display.feed(data)
            current_time_stamp = data["timestamp"]

    finally:
        display.quit()


def run_video_tester(argv=None):

    config, argv = get_scor_config(argv=argv)
    if len(argv) > 1 and argv[1] in ('-h', '--help'):
        # TODO improve the help message
        sys.exit(1)
    team = u'top'
    print argv
    if len(argv) > 1:
        if argv[1] == u'north':
            team = argv[1]
        elif argv[1] == u'south':
            team = argv[1]
    config.set_default_teams()
    video = get_proxy(config, "video", team)

    display = TkVideoDisplay(video)
    display.mainloop()

def run_serial_tester(argv=None):

    config, argv = get_scor_config(argv=argv)

    if len(argv) > 1 and argv[1] in ('-h', '--help'):
        # TODO improve the help message
        sys.exit(1)

    config.set_default_teams()

    robot_id = int(argv[1])
    robot_section = ROBOT_SECTION_MASK % robot_id
    serial_port = config.get(robot_section, "port")
    serial = get_proxy(config, "serial", serial_port)

    try:
        while 1:
            line = raw_input(">>> ")
            serial.write(line)
    finally:
        serial.write("D,0,0")
