# -*- coding:utf-8 -*-
"""
Test cases for the control interface.
"""

from unittest import TestCase, main as unittest_main

from scor.pion import control, wrapper

class Control_TestCase(TestCase):
    """
    Runs tests on the control interface.
    """

    def setUp(self):
        """
        Creates a control interface for two Kheperas.
        """
        # vars
        self.ports = [u"/dev/ttyS0", u"/dev/stdout"]
        self.speeds = [-1, 1]
        # build kheps' list
        self.khep_list = {}
        for port in self.ports:
            self.khep_list[self.ports.index(port)] = wrapper.Wrapper(port)
        # create the control
        self.control = control.Control(self.khep_list)

    def tearDown(self):
        """
        Deletes the control interface.
        """
        del self.control

    def test_registration(self):
        """
        Tests methods related to the registration process.
        """
        self.assertEqual(len(self.control._khep_list), len(self.khep_list))
        for i in xrange(len(self.khep_list)):
            self.assertEqual(self.control._khep_list[i].port, self.ports[i])

    def test_move(self):
        """
        Tests move() method.
        """
        for i in xrange(len(self.khep_list)):
            self.control.move(i, self.speeds[i])
            self._compare_speeds(i, control.translate_speed(self.speeds[i]), \
                control.translate_speed(self.speeds[i]))

    def test_rotate(self):
        """
        Tests rotate() method.
        """
        for i in xrange(len(self.khep_list)):
            self.control.rotate(i, self.speeds[i])
            self._compare_speeds(i, control.translate_speed(self.speeds[i]), \
                control.translate_speed(-self.speeds[i]))

    def test_stop(self):
        """
        Tests stop() method.
        """
        for i in xrange(len(self.khep_list)):
            self.control.stop(i)
            self._compare_speeds(i, 0, 0)

    def test_set_speed(self):
        """
        Tests set_speed() method.
        """
        for i in xrange(len(self.khep_list)):
            self.control.set_speed(i, self.speeds[i], self.speeds[i] + 12)
            self._compare_speeds(i, control.translate_speed(self.speeds[i]), \
                control.translate_speed(self.speeds[i] + 12))

    def _compare_speeds(self, khepera_id, left_speed, right_speed):
        """
        Compare passed speeds with the ones from the Khepera.
        """
        self.assertEqual(
            self.control._khep_list[khepera_id].queue.get_nowait(),
            control.MOVE_MASK % (left_speed, right_speed))

if __name__ == '__main__':
    unittest_main()

# vim: ts=4
