# -*- coding:utf-8 -*-
"""
Test cases for the control interface's manager.
"""
from unittest import TestCase, main as unittest_main
import os, sys
import Pyro.core
from threading import Thread

from scor.common.config import ROBOT_SECTION_MASK
from scor.pion import manager


class Manager_TestCase(TestCase):
    """
    Runs tests on the control interface's manager.
    """

    def setUp(self):
        """
        Write an appropriated custom.ini config file.
        If custom.ini already exists, save it.
        Then creates a manager using this config file.
        """
        # vars
        self.pyro_server = None
        self.ports = [u"/dev/ttyS0", u"/dev/stdout"]
        self.speeds = [-1, 1]

        self.config = os.path.expanduser("~/.scor")
        self.backup = os.path.expanduser("~/.scor.bak")
        if os.path.exists(self.config):
            os.rename(self.config, self.backup)

        # Setting the parameters of the config file.
        # Keys of params are sections names, values are dict, whose keys
        # are config keys, and values are config value.
        params = {}

        for i in xrange(len(self.ports)):
            params[ROBOT_SECTION_MASK % i] = {
                u"port" : self.ports[i]
            }

        # set_default_teams() registers lots of Kheperas, disable them
        for i in xrange(len(self.ports), 5):
            params[ROBOT_SECTION_MASK % i] = {
                u"enabled" : u"0"
            }

        cfg = open(self.config, u"w")
        for section, options in params.items():
            cfg.write(u"[" + section + u"]\n")
            for key, value in options.items():
                cfg.write(key + u"=" + value + u"\n")

        cfg.close()

        # ha, this is ugly \o/
        sys.argv.append(u"--debug")

        self.manager = manager.Manager()

    def tearDown(self):
        """
        Deletes the manager, and restores the config file.
        """
        # stop the Pyro server
        if self.pyro_server is not None:
            self.manager._pyro_server.shutdown(True)

        # delete the manager
        del self.manager

        # ha, this is ugly \o/²
        sys.argv.remove(u"--debug")

        os.remove(self.config)
        if os.path.exists(self.backup):
            os.rename(self.backup, self.config)

    def test_registration(self):
        """
        Tests methods related to the registration process.
        """
        khep_list = self.manager.get_kheps()
        self.assertEqual(len(khep_list), len(self.ports))
        for i in xrange(len(self.ports)):
            self.assertEqual(khep_list[i].port, self.ports[i])

    def test_pyro(self):
        """
        Tests the Pyro server.
        """
        # start the server in another thread
        self.pyro_server = Thread(target=self.manager.start)
        self.pyro_server.start()
        host = self.manager.config.get("pyro_server","control_host")
        port = self.manager.config.getint("pyro_server","control_port")
        uri = u"PYROLOC://%s:%i/control" % (host, port)
        control = Pyro.core.getProxyForURI(uri)
        # test whether the remote object is ok
        control.stop(1)

if __name__ == '__main__':
    unittest_main()

# vim: ts=4
