# -*- coding:utf-8 -*-
"""
Test cases for the serial wrapper.
"""

from unittest import TestCase, main as unittest_main

from scor.pion import wrapper

class Wrapper_Fake_TestCase(TestCase):
    """
    Runs tests on the Wrapper.
    """

    def setUp(self):
        """
        Creates a fake serial connection.
        """
        # vars
        self.port = u"/dev/null"
        self.messages = ['D,-1,5\n', 'D,3,2\n', 'D,4,-6\n', 'D,-7,-8\n']
        # creation
        self.cnct = wrapper.Wrapper(self.port)

    def tearDown(self):
        """
        Deletes the fake serial connection.
        """
        del self.cnct

    def test_init_configure(self):
        """
        Tests attributes value.
        """
        self.assertEqual(self.cnct.port, self.port)

    def test_write_one(self):
        """
        Sends a message to the serial line and checks its value.
        """
        self.cnct.write(self.messages[1])
        self.assertEqual(self.cnct.queue.get_nowait(), self.messages[1])

    def test_write_two(self):
        """
        Sends two messages to the serial line and checks their value.
        """
        for i in xrange(2):
            self.cnct.write(self.messages[i])
            self.assertEqual(self.cnct.queue.get_nowait(), self.messages[i])

    def test_write_size(self):
        """
        Sends several messages to the serial line and checks if they are all in
        the queue.
        """
        for i in reversed(xrange(4)):
            self.cnct.write(self.messages[i])
        self.assertEqual(self.cnct.queue.qsize(), 4)

if __name__ == '__main__':
    unittest_main()

# vim: ts=4
